/*
 * Sections of this code lifted from the Android tutorial's
 * Notepadv2 project provided by Google.
 */

package edu.berkeley.cs160.alarmbuddy;

import java.io.FileNotFoundException;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

/**
 * Simple notes database access helper class. Defines the basic CRUD operations
 * for the notepad example, and gives the ability to list all notes as well as
 * retrieve or modify a specific note.
 * 
 * This has been improved from the first version of this tutorial through the addition
 * of better error handling and also using returning a Cursor instead of using
 * a collection of inner classes (which is less scalable and not recommended).
 */
public class AlarmsDbAdapter {

	private static final String DATABASE_NAME = "data";
    private static final String DATABASE_TABLE = "notes";
    private static final int DATABASE_VERSION = 2;
	
	/* Variables for each key in the DB */
	public static final String KEY_ROWID="_id";
    public static final String KEY_TITLE="title";
    public static final String KEY_TIME="time";
    public static final String KEY_ACTIVE="active";
    public static final String KEY_RECURRENCE="recurrence";
    public static final String KEY_CUSTOM_RECURRENCE="cust_recurrence";
    public static final String KEY_TONE="tone";
    public static final String KEY_IS_TIMER="is_timer";
    
    /** Database creation sql statement */
    private static final String DATABASE_CREATE =
        "create table " + DATABASE_TABLE + " ("
        	+ KEY_ROWID + " integer primary key autoincrement, "
            + KEY_TITLE + " text not null, " //must have value input
            + KEY_TIME + " integer, " //defaults to null
            + KEY_ACTIVE + " integer DEFAULT 1, " //default ON
            + KEY_RECURRENCE + " text DEFAULT 'none', " //default 'none'
            + KEY_CUSTOM_RECURRENCE + " integer DEFAULT 0, "
            + KEY_TONE + " text, " //defaults to null
            + KEY_IS_TIMER + " integer DEFAULT 0" //defaults to null
        + ");";
    
    private SQLiteDatabase mDb;
    private final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public AlarmsDbAdapter(Context ctx) {
        this.mCtx = ctx;
    }
    
    /**
     * Open the alarms database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public AlarmsDbAdapter open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(DATABASE_CREATE);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() {
        mDb.close();
    }

    /**
     * Create a new alarm using the title and body provided. If the note is successfully created
     * return the new rowId for that note, otherwise return a -1 to indicate failure.
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    public long createAlarm(String title) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_TITLE, title);
        initialValues.put(KEY_IS_TIMER, 0);
        return mDb.insert(DATABASE_TABLE, null, initialValues);
    }

    /**
     * Create a new alarm using the title and body provided. If the note is successfully created
     * return the new rowId for that note, otherwise return a -1 to indicate failure.
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    public long createTimer(String title) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_TITLE, title);
        initialValues.put(KEY_IS_TIMER, 1);
        initialValues.put(KEY_TIME, 5);
        return mDb.insert(DATABASE_TABLE, null, initialValues);
    }
    
    /**
     * Delete the alarm with the given rowId
     * @param rowId id of alarm to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteAlarm(long rowId) {
        return mDb.delete(DATABASE_TABLE, KEY_ROWID + "=" + rowId, null) > 0;
    }

    /**
     * Return a Cursor over the list of all notes in the database
     * @return Cursor over all notes
     */
    public Cursor fetchAllAlarms() {
        return mDb.query(DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_TIME, 
                KEY_ACTIVE, KEY_RECURRENCE, KEY_CUSTOM_RECURRENCE, 
                KEY_TONE, KEY_IS_TIMER,}, null, null, null, null, null);
    }

    /**
     * Return a Cursor positioned at the alarm that matches the given rowId
     * @param rowId id of alarm to retrieve
     * @return Cursor positioned to matching alarm, if found
     * @throws SQLException if alarm could not be found/retrieved
     */
    public Cursor fetchAlarm(long rowId) throws SQLException {
        Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_TIME, 
                KEY_ACTIVE, KEY_RECURRENCE, KEY_CUSTOM_RECURRENCE, 
                KEY_TONE, KEY_IS_TIMER,}, KEY_ROWID + "=" + rowId, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) {
            throw new SQLException("No note matching ID: " + rowId);
        }
        return result;
    }

    /**
     * Update the alarm using the details provided. The alarm to be updated is specified using
     * the rowId, and it is altered to use arguments passed in
     * @param rowId id of note to update
     * @return true if the note was successfully updated, false otherwise
     */
    public boolean updateAlarm(long rowId, String title, int time, int active, 
    		String recurrence, int custom_recurrence, String tone, int isTimer) {
        ContentValues args = new ContentValues();
        args.put(KEY_TITLE, title);
        Log.println(100, "print", "AlarmsDbAdapter: updated title="+title);
        int x = mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null);
        Log.println(100, "print", "DbAdapter: updating just title result for id="+rowId+": "+x);
        
        args.put(KEY_TIME, time);
        args.put(KEY_ACTIVE, active);
        args.put(KEY_RECURRENCE, recurrence);
        args.put(KEY_CUSTOM_RECURRENCE, custom_recurrence);
        args.put(KEY_TONE, tone);
        args.put(KEY_IS_TIMER, isTimer);
        
        return mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null) > 0;
    }
    
}
