/**
 * InstalledMiniAppDbAdapter.java
 * @author Mike Ross
 * 
 * A super simple Database adapter to store all the installed MiniApps, and the
 * names of the database tables in which they reside.
 * Only one such table should exist within the program.
 *
 * This makes adding MiniApps to the AddMiniApps screen just a matter
 * of a few lines of code.
 * 
 * Sections of this code lifted from the Android tutorial's
 * Notepadv2 project provided by Google.
 * 
 * 
 */

package edu.berkeley.cs160.alarmbuddy;

import java.io.FileNotFoundException;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

public class InstalledMiniAppDbAdapter {

    private static final String DATABASE_NAME = "data";
    private static final String DATABASE_TABLE = "installed_mini_apps";
    private static final int DATABASE_VERSION = 2;
    
    /* Variables for each key in the DB */
    public static final String KEY_ROWID="_id";
    
    // the name of the database table the MiniApp instances are stored in
    public static final String KEY_TABLE_NAME="table_name";
    
    // a nice name to label the MiniApps in the Add Mini Apps screen
    public static final String KEY_NICE_NAME="nice_name";
    
    // the icon for this particular miniapp
    public static final String KEY_ICON="icon";
    
    /** Database creation sql statement */
    private static final String DATABASE_CREATE =
        "create table if not exists " + DATABASE_TABLE + " ("
            + KEY_ROWID + " integer primary key autoincrement, "
            //+ KEY_TABLE_NAME + " text not null, " //must have value input
            + KEY_NICE_NAME + " text not null, " //must have value input
            + KEY_ICON + " integer DEFAULT 0"
        + ");";
    
    private SQLiteDatabase mDb;
    protected final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public InstalledMiniAppDbAdapter(Context ctx) {
        this.mCtx = ctx;
    }
    
    /**
     * Open the Installed MiniApps database. If it cannot be opened, try to
     * create a new instance of the database. If it cannot be created, throw
     * an exception to signal the failure
     * 
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public InstalledMiniAppDbAdapter open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
            deleteTable();
            mDb.execSQL(DATABASE_CREATE);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(DATABASE_CREATE);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() {
        mDb.close();
    }

    /**
     * Clear the database of all entries
     */
    public void deleteAllMiniApps() {
    	// pseudo-infinite lists always scare me.
    	for(Cursor c = fetchAllMiniApps(); true; c.next()){
    		deleteMiniApp(c.getLong(c.getColumnIndex(KEY_ROWID)));
    		if(c.isLast()) {
    			break;
    		}
    	}
    }
    
    /**
     * Delete the whole table.  This is unrecoverable and only for internal
     * use.
     */
    public void deleteTable() {
    	if(mDb == null) return;
    	mDb.execSQL("drop table if exists " + DATABASE_TABLE);
    	Log.println(100, "InstalledMiniAppDbAdapter", "Table dropped");
    }
    
    /**
     * Create a new MiniApp with the provided name provided. If the MiniApp is	
     * successfully created, return the new rowId for that MiniApp, otherwise
     * return a -1 to indicate failure.
     * @param name the name of the MiniApp
     * @return rowId or -1 if failed
     */
    public long installMiniApp(MiniAppActivity m) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_NICE_NAME, m.getNiceName());
        initialValues.put(KEY_ICON, m.getIconPath());
        return mDb.insert(DATABASE_TABLE, null, initialValues);
    }

    /**
     * Delete the MiniApp with the given rowId
     * @param rowId id of alarm to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteMiniApp(long rowId) {
        return mDb.delete(DATABASE_TABLE, KEY_ROWID + "=" + rowId, null) > 0;
    }

    /**
     * Return a Cursor over the list of all MiniApps in the database
     * @return Cursor over all MiniApps
     */
    public Cursor fetchAllMiniApps() {
        return mDb.query(DATABASE_TABLE, new String[] {
                KEY_ROWID,
                //KEY_TABLE_NAME,
                KEY_NICE_NAME,
                KEY_ICON},
                null, null, null, null, null);
    }

    /**
     * Return a Cursor positioned at the MiniApp that matches the given rowId
     * @param rowId id of alarm to retrieve
     * @return Cursor positioned to matching alarm, if found
     * @throws SQLException if alarm could not be found/retrieved
     */
    public Cursor fetchMiniApp(long rowId) throws SQLException {
        Cursor result = mDb.query(
        	true,
        	DATABASE_TABLE,
        	new String[] {
               	KEY_ROWID,
               	//KEY_TABLE_NAME,
               	KEY_NICE_NAME,
               	KEY_ICON,},
            KEY_ROWID + " = " + rowId,
            null, null, null, null);
        if ((result.count() == 0) || !result.first()) {
            throw new SQLException("No MiniApp matching ID: " + rowId);
        }
        return result;
    }
    
    
    /**
     * Return whether or not a MiniApp of the given name is installed 
     * @param name The name to look for
     * @return true if the MiniApp is installed, false otherwise
     */
    public boolean miniAppIsInstalled(String name) {
    	Cursor result = mDb.query(DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_NICE_NAME, KEY_ICON,},
                KEY_NICE_NAME + " = '" + name + "'",
                null, null, null, null);
    	return (result.count() > 0);
    }

    /**
     * Update the MiniApp using the details provided. The alarm to be updated is specified using
     * the rowId, and it is altered to use arguments passed in
     * @param rowId id of MiniApp to update
     * @return true if the MiniApp was successfully updated, false otherwise
     */
    /*
    public boolean updateAlarm(long rowId, String title, int time, int active, 
            String recurrence, int custom_recurrence, String tone, int miniapps) {
        ContentValues args = new ContentValues();
        args.put(KEY_TABLE_NAME, title);
        
        return mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null) > 0;
    }*/
}
