package edu.berkeley.cs160.alarmbuddy;

import java.io.FileNotFoundException;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

/**
 * WeatherInstanceDb.java
 * @author Mike Ross
 *
 * A database storing all the instance ids of MiniApps for a particular alarm.
 * An alarm will look at this table, find the MiniApp class based upon an entry
 * in the table, then look up the MiniApp instance of the given id for that
 * class
 */
public class MiniAppDb {
    
	private static final String DATABASE_NAME = "data";
	private static final int DATABASE_VERSION = 2;
	
	
	/* Fields for each key in the DB */
	private static final String KEY_ROWID = "_id";
	public static final String KEY_MINIAPP_NAME = "miniapp_name";
	public static final String KEY_ALARM_ID = "alarm_id";
	private static SQLiteDatabase mDb;
	private Context mCtx;
	private static String DATABASE_TABLE = "mini_apps_per_alarm_table";
	
	/** the alarm id used per miniAppDb instance.  This database actually
	 * stores miniApp instances for ALL alarms, but this refers to the alarm
	 * that is currently accessing it.*/ 
	private long mAlarmId;
		
	private static final String databaseCreate =
		"create table if not exists " + DATABASE_TABLE + " ("
		+ KEY_ROWID + " integer primary key autoincrement,"
		+ KEY_MINIAPP_NAME + " text not null, "
		+ KEY_ALARM_ID + " integer "
		+ ");";
		
    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public MiniAppDb(Context ctx, long alarmId) {
        this.mCtx = ctx;
        this.mAlarmId = alarmId;
    }
    
    /**
     * Open the MiniApp database. If it cannot be opened, try to
     * create a new instance of the database. If it cannot be created, throw
     * an exception to signal the failure
     * 
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public MiniAppDb open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
            mDb.execSQL(databaseCreate);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(databaseCreate);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }
    
    public void close() {
        mDb.close();
    }
    
    
    /**
     * Create a new Mini App row using the title and body provided.
     *  If the row is successfully created, return the new rowId,
     *  otherwise return a -1 to indicate failure.
     *  For public access, use updateInstance
     * @param miniAppName the nice name of the miniApp being created
     * @param instanceId the id to use to look up the given miniApp
     * @return rowId or -1 if failed
     */
    private long addInstance(String miniAppName) {
        ContentValues values = new ContentValues();
        values.put(KEY_MINIAPP_NAME, miniAppName);
        values.put(KEY_ALARM_ID, mAlarmId);
        return mDb.insert(DATABASE_TABLE, null, values);
    }
    
    
    
    /**
     * Update the row using the details provided. The entry to be updated is
     * specified using alarmId and MiniAppName, and it is altered to use arguments passed in
     * @return true if the note was successfully updated, false otherwise
     */
    public boolean updateInstance(String miniAppName) {
    	if(instanceExists(miniAppName)) {
    		ContentValues args = new ContentValues();
            args.put(KEY_ALARM_ID, mAlarmId);
            args.put(KEY_MINIAPP_NAME, miniAppName);
            return mDb.update(DATABASE_TABLE, args,
            		KEY_ALARM_ID + "=" + mAlarmId + " AND " 
            		+ KEY_MINIAPP_NAME + "= '" + miniAppName + "'",
            		null) > 0;	
    	}
    	return -1 != addInstance(miniAppName);   
    }
    
    /**
     * return true iff an instance of the given miniApp exists for the given
     * alarmId
     * @param alarmId
     * @param miniAppName
     * @return
     * @throws SQLException
     */
    public boolean instanceExists(String miniAppName) throws SQLException {
        Cursor c = mDb.query(DATABASE_TABLE, null,
        		KEY_ALARM_ID + "=" + mAlarmId + " AND " 
        		+ KEY_MINIAPP_NAME + "= '" + miniAppName + "'",
                null, null, null, null);
        return !( (c.count() == 0) || !c.first());
    }
    
    /**
     * Delete the miniApp instance with the given rowId.  Also signals to the
     * MiniApp that it can delete any data about this instance.
     * @param rowId
     * @return
     */
    public int deleteInstance(String miniAppName) {
    	Cursor c = fetchMiniAppCursorByAlarmId();
    	MiniAppActivity m = MiniAppActivity.miniAppFromString(miniAppName);
    	m.deleteInstance(mAlarmId);
    	return mDb.delete(DATABASE_TABLE,
    			KEY_ALARM_ID + "=" + mAlarmId + " AND " 
        		+ KEY_MINIAPP_NAME + "= '" + miniAppName + "'",
    			null);
    }
    
    
    public Cursor fetchMiniAppCursorByRowId(long rowId) throws SQLException {
        Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ROWID + "=" + rowId,
                null, null, null, null);
        if ((c.count() == 0) || !c.first()) {
            throw new SQLException("No MiniAppDb row with row ID: " + rowId);
        }
        return c;
    }
    
    public Cursor fetchMiniAppCursorByAlarmId() {
        Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ALARM_ID + "=" + mAlarmId,
                null, null, null, null);
        Log.println(100, "wtf", "wtf: fetchMiniAppCursorByAlarmId");
        Log.println(100, "wtf", "wtf: .. mALarmId: " + mAlarmId + ", count: " + c.count());
        return c;
    }
    
    /**
     * Return an array of MiniApp instances for the current alarm.
     * @return
     */
    public MiniAppActivity[] getMiniAppArray(Context ctx) {

    	
    	Cursor c = fetchMiniAppCursorByAlarmId();
    	int count = c.count();
    	c.first();
    	int i = c.position();
    	MiniAppActivity[] apps = new MiniAppActivity[count];
    	while(i < count) {
    		String appName = c.getString(c.getColumnIndex(KEY_MINIAPP_NAME));
    		apps[i] = MiniAppActivity.miniAppFromString(appName);
    		c.next();
    		i++;
    	}
    	
    	return apps;
    }
    
    
    
    public int getIcon() {
        return R.drawable.test_icon;
    }
    
    
    public String getInstanceTableName() {
        return DATABASE_TABLE;
    }
}

