package edu.berkeley.cs160.alarmbuddy;

/**
 * 	The Notepad Database Adapter from Notepad example.
 * 	Unchanged.. yet.
 * 	-- andry jong
 */

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;

import java.io.FileNotFoundException;

/**
 * Simple notes database access helper class. Defines the basic CRUD operations
 * for the notepad example, and gives the ability to list all notes as well as
 * retrieve or modify a specific note.
 * 
 * This has been improved from the first version of this tutorial through the addition
 * of better error handling and also using returning a Cursor instead of using
 * a collection of inner classes (which is less scalable and not recommended).
 */


public class ToDoDbAdapter{

    public static final String KEY_TITLE="title";
    public static final String KEY_BODY="body";
    public static final String KEY_ROWID="_id";
    public static final String KEY_ALARM_ID = "alarm_id";
    public static final String NICE_NAME="ToDo List";
    public static final String DATABASE_NAME = "data";
    public static final String DATABASE_TABLE = "todo_inner_table";
    
    private long mAlarmId = -1;
    
    /**
     * Database creation sql statement
     */
    private static final String DATABASE_CREATE =
        "create table if not exists " + DATABASE_TABLE + " ("
        + KEY_ROWID + " integer primary key autoincrement, "
        + KEY_TITLE + " text not null, "
        + KEY_ALARM_ID + " integer, "
        + KEY_BODY + " text not null);";
        
    private static final int DATABASE_VERSION = 2;

    private SQLiteDatabase mDb;
    private final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public ToDoDbAdapter(Context ctx, long alarmId) {
        this.mCtx = ctx;
        this.mAlarmId = alarmId;
    }
    
    /**
     * Open the notes database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public ToDoDbAdapter open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
            mDb.execSQL(DATABASE_CREATE);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(DATABASE_CREATE);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() {
        mDb.close();
    }

    /**
     * Create a new note using the title and body provided. If the note is successfully created
     * return the new rowId for that note, otherwise return a -1 to indicate failure.
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    public long createNote(String title, String body) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_TITLE, title);
        initialValues.put(KEY_BODY, body);
        initialValues.put(KEY_ALARM_ID, mAlarmId);
        return mDb.insert(DATABASE_TABLE, null, initialValues);
    }

    /**
     * Delete the note with the given rowId
     * @param rowId id of note to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteNote(long rowId) {
        return mDb.delete(DATABASE_TABLE,
        		KEY_ROWID + "=" + rowId + " AND " + KEY_ALARM_ID + "=" + mAlarmId,
        		null) > 0;
    }

    /**
     * Return a Cursor over the list of all notes in the database
     * @return Cursor over all notes
     */
    public Cursor fetchAllNotes() {
        return mDb.query(DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_BODY},
                KEY_ALARM_ID + "=" + mAlarmId,
                null, null, null, null);
    }

    /**
     * Return a Cursor positioned at the note that matches the given rowId
     * @param rowId id of note to retrieve
     * @return Cursor positioned to matching note, if found
     * @throws SQLException if note could not be found/retrieved
     */
    public Cursor fetchNote(long rowId) throws SQLException {
        Cursor result = mDb.query(true, DATABASE_TABLE, new String[] {
                KEY_ROWID, KEY_TITLE, KEY_BODY},
                KEY_ROWID + "=" + rowId + " AND " + KEY_ALARM_ID + "=" + mAlarmId, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) {
            throw new SQLException("No note matching rowID/alarmId: " + rowId + "/" + mAlarmId);
        }
        return result;
    }

    /**
     * Update the note using the details provided. The note to be updated is specified using
     * the rowId, and it is altered to use the title and body values passed in
     * @param rowId id of note to update
     * @param title value to set note title to
     * @param body value to set note body to
     * @return true if the note was successfully updated, false otherwise
     */
    public boolean updateNote(long rowId, String title, String body) {
        ContentValues args = new ContentValues();
        args.put(KEY_TITLE, title);
        args.put(KEY_BODY, body);
        return mDb.update(DATABASE_TABLE, args,
        		KEY_ROWID + "=" + rowId + " AND " + KEY_ALARM_ID + "=" + mAlarmId,
        		null) > 0;
    }
    
    public ToDoDbAdapter fetchMiniApp(long rowId) throws SQLException {
    	return null;
    }


    
    public String getDbCreateString() {
    	return DATABASE_CREATE;
    }
}
