package edu.berkeley.cs160.alarmbuddy;

import java.io.FileNotFoundException;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

/**
 * Browser.java
 * @author Mike Ross
 *
 * Alarm Browser Mini Application
 *  
 * This will eventually provide an example for how to create a MiniApp.
 * For now, it's lacking methods.
 */
public class WeatherDb {
	
	private static final String KEY_ROWID = "_id";
	private static final String KEY_ALARM_ID = "alarm_id";
	private static final String KEY_ZIPCODE = "zipcode";
	private static final String DATABASE_NAME = "data";
	private static final String DATABASE_TABLE = "weather_mini_app";
	private static final int	DATABASE_VERSION = 2;
	
	private static final String NICE_NAME = "Weather";
	private static SQLiteDatabase mDb;
	private Context mCtx;
	
	/* to be set per class */
	protected String getDbCreateString() {
		return "create table if not exists " + DATABASE_TABLE + " ("
		+ KEY_ROWID + " integer primary key autoincrement, "
		+ KEY_ZIPCODE + " text not null, "
		+ KEY_ALARM_ID + " integer "
		+ ");";
	}
	
	/**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public WeatherDb(Context ctx) {
        this.mCtx = ctx;
    }
    
    public void close() {
        mDb.close();
    }
    
    /**
     * Open the MiniApp's database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    protected WeatherDb open() throws SQLException {
        try { 
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
            
            mDb.execSQL(getDbCreateString());
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(getDbCreateString());
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
        
    }
    
    /**
     * Create a new Weather Mini App row using the title and body provided.
     *  If the row is successfully created, return the new rowId,
     *  otherwise return a -1 to indicate failure.
     *  Use "updateInstance" for non-private access
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    private long addInstance(long alarmId, String zipCode) {
    	ContentValues values = new ContentValues();
    	values.put(KEY_ZIPCODE, zipCode);
    	values.put(KEY_ALARM_ID, alarmId);
    	return mDb.insert(DATABASE_TABLE, null, values);
    }
    
    
    public Cursor fetchMiniAppCursor(long alarmId) throws SQLException {
    	Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ALARM_ID + "=" + alarmId,
    			null, null, null, null);
    	if ((c.count() == 0) || !c.first()) {
    		throw new SQLException("No Weather Instance with row ID: " + alarmId);
    	}
    	return c;
    }
    
    /**
     * return true iff an entry with this alarmId exists already.
     * @param alarmId
     * @return
     */
    public boolean alarmIdExists(long alarmId) {
    	Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ALARM_ID + "=" + alarmId,
    			null, null, null, null);
    	if ((c.count() == 0) || !c.first()) return false;
    	return true;
    }
    
    /**
     * Return the zipcode for weather instance for the given alarm Id.
     * @param alarmId
     * @return
     */
    public String getInstanceZipcode(long alarmId) {
    	Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ALARM_ID + "=" + alarmId,
    			null, null, null, null);
    	if ((c.count() == 0) || !c.first()) {
    		return "";
    	}
    	return c.getString(c.getColumnIndex(KEY_ZIPCODE));    	
    }
    
    public String getZipCode(long alarmId) {
    	Cursor c = fetchMiniAppCursor(alarmId);
    	return c.getString(c.getColumnIndex(KEY_ZIPCODE));
    }
    
    /**
     * Update the entry using the details provided. The row to be updated is
     * specified using the alarmId, and it is altered to use arguments passed in
     * @return true if the note was successfully updated, false otherwise
     */
    public boolean updateInstance(long alarmId, String zipCode) {
    	
    	if(alarmIdExists(alarmId)) {
    		ContentValues args = new ContentValues();
            args.put(KEY_ALARM_ID, alarmId);
            args.put(KEY_ZIPCODE, zipCode);
            return mDb.update(DATABASE_TABLE, args, KEY_ALARM_ID + "=" + alarmId, null) > 0;	
    	}
    	return -1 != addInstance(alarmId, zipCode);   
    }

    /**
     * Delete the alarm with the given rowId
     * @param rowId id of alarm to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteInstance(long alarmId) {
        return mDb.delete(DATABASE_TABLE, KEY_ALARM_ID + "=" + alarmId, null) > 0;
    }
}

